#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <minix/config.h>
#include <minix/const.h>
#include <minix/type.h>
#include "../kernel/const.h"
#include "../kernel/type.h"
#include "../kernel/proc.h"

#undef printf


void dump_out(int fd);
void dump_seg(struct mem_map mp_seg[NR_SEGS], int seg, int fd);
void error(char *s);

char *prog;


int main(argc, argv)
int argc;
char **argv;
{
  char *file;
  int fd;

  stdout->_flags &= ~PERPRINTF;
  prog = argv[0];
  if (argc > 2 || argc > 1 && argv[1][0] == '-')
  {
	fprintf(stderr, "Usage: %s [corefile]\n", prog);
	exit(1);
  }
  if (argc == 2)
	file = argv[1];
  else
	file = "core";

  if ((fd = open(file, O_RDONLY)) < 0)
	error(prog);

  dump_out(fd);

  close(fd);
  return(0);
}


void dump_out(fd)
int fd;
{
  static struct mem_map mp_seg[NR_SEGS];
  static struct proc proc;

  if (read(fd, mp_seg, sizeof(mp_seg)) != sizeof(mp_seg))
	error("reading segment list");
  if (read(fd, &proc, sizeof(proc)) != sizeof(proc))
	error("reading proc entry");

  printf("text  segment:  phys = %08x, vir = %08x, len = %08x\n",
	mp_seg[T].mem_phys << CLICK_SHIFT,
	mp_seg[T].mem_vir  << CLICK_SHIFT,
	mp_seg[T].mem_len  << CLICK_SHIFT);
  printf("data  segment:  phys = %08x, vir = %08x, len = %08x\n",
	mp_seg[D].mem_phys << CLICK_SHIFT,
	mp_seg[D].mem_vir  << CLICK_SHIFT,
	mp_seg[D].mem_len  << CLICK_SHIFT);
  printf("stack segment:  phys = %08x, vir = %08x, len = %08x\n",
	mp_seg[S].mem_phys << CLICK_SHIFT,
	mp_seg[S].mem_vir  << CLICK_SHIFT,
	mp_seg[S].mem_len  << CLICK_SHIFT);

  printf("\nR0  = %08x, R1  = %08x, R2  = %08x, R3  = %08x\n",
	proc.p_reg.retreg, proc.p_reg.r1, proc.p_reg.r2, proc.p_reg.r3);
  printf("R4  = %08x, R5  = %08x, R6  = %08x, R7  = %08x\n",
	proc.p_reg.r4, proc.p_reg.r5, proc.p_reg.r6, proc.p_reg.r7);
  printf("R8  = %08x, R9  = %08x, R10 = %08x, R11 = %08x\n",
	proc.p_reg.r8, proc.p_reg.r9, proc.p_reg.r10, proc.p_reg.r11);
  printf("R12 = %08x, R13 = %08x, R14 = %08x, PC  = %08x\n",
	proc.p_reg.sp, proc.p_reg.sl, proc.p_reg.lk, proc.p_reg.pc);

  printf("R8_fir  = %08x, R9_fir  = %08x, R10_fir = %08x, R11_fir = %08x\n",
	proc.p_reg.r8_firq, proc.p_reg.r9_firq,
	proc.p_reg.r10_firq, proc.p_reg.r11_firq);
  printf("R12_fir = %08x, R13_fir = %08x, R14_fir = %08x\n",
	proc.p_reg.r12_firq, proc.p_reg.sp_firq, proc.p_reg.lk_firq);

  printf("R13_irq = %08x, R14_irq = %08x\n",
	proc.p_reg.sp_irq, proc.p_reg.lk_irq);

  printf("R13_svc = %08x, R14_svc = %08x\n",
	proc.p_reg.sp_svc, proc.p_reg.lk_svc);

  dump_seg(mp_seg, T, fd);
  dump_seg(mp_seg, D, fd);
  dump_seg(mp_seg, S, fd);
}


void
dump_seg(mp_seg, seg, fd)
struct mem_map mp_seg[NR_SEGS];
int seg, fd;
{
  static char *seg_name[NR_SEGS] = {"Text", "Data", "Stack"};
  static char buf[CLICK_SIZE];
  int nr_clicks, i, j, addr;

  if (mp_seg[seg].mem_len == 0) return;
  printf("\n%s segment:\n", seg_name[seg]);
  addr = mp_seg[seg].mem_vir << CLICK_SHIFT;
  nr_clicks = mp_seg[seg].mem_len;

  while (nr_clicks--)
  {
	if (read(fd, buf, CLICK_SIZE) != CLICK_SIZE)
		error("reading click");
	for (i = 0; i < CLICK_SIZE; i += 16, addr += 16)
	{
		printf("%08x  ", addr);
		for (j = i; j < i + 16; j++)
			printf("%02x ", buf[j]);
		printf(" ");
		for (j = i; j < i + 16; j++)
			printf("%c", buf[j] < ' ' ? '.' : buf[j]);
		printf("\n");
	}
  }
}


void
error(s)
char *s;
{
  perror(s);
  exit(1);
}
